package com.sxtvs.open.api.news.service.impl;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.SortOptions;
import co.elastic.clients.elasticsearch._types.SortOrder;
import co.elastic.clients.elasticsearch._types.query_dsl.*;
import co.elastic.clients.elasticsearch.core.BulkRequest;
import co.elastic.clients.elasticsearch.core.SearchRequest;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import co.elastic.clients.elasticsearch.core.bulk.BulkOperation;
import co.elastic.clients.json.JsonData;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.api.news.dto.DataRequestDTO;
import com.sxtvs.open.api.news.entity.CategoryData;
import com.sxtvs.open.api.news.entity.YoumeiData;
import com.sxtvs.open.api.news.entity.YoumeiEsData;
import com.sxtvs.open.api.news.entity.YoumeiOffset;
import com.sxtvs.open.api.news.mapper.YoumeiDataMapper;
import com.sxtvs.open.api.news.service.IYoumeiDataService;
import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.es.EsClientPool;
import com.sxtvs.open.core.sls.AliyunLogger;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.TextUtils;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-02-20
 */
@Service
@Slf4j
public class YoumeiDataServiceImpl extends ServiceImpl<YoumeiDataMapper, YoumeiData> implements IYoumeiDataService {
//    @Resource
//    private ElasticsearchClient elasticsearchClient;

    @Resource
    private YoumeiOffsetServiceImpl youmeiOffsetService;

    @Resource
    private AliyunLogger logger;

    @Resource
    private EsClientPool esClientPool;

    private Boolean lock = false;

    public Page<YoumeiEsData> search(DataRequestDTO dataRequestDTO) throws Exception {

        ElasticsearchClient elasticsearchClient = esClientPool.getClient();
        SearchResponse<YoumeiEsData> response = null;
        Page<YoumeiEsData> page = new Page<>(dataRequestDTO.getPage(), dataRequestDTO.getPageSize());
        int offset = (int) ((page.getCurrent() - 1) * page.getSize());
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        try {
            SearchRequest.Builder builder = new SearchRequest.Builder();
            builder.index("news_data").from(offset).size((int) page.getSize());
            Query query = Query.of(q -> q.bool(b -> {
                // 日期
                if (dataRequestDTO.getStart() != null) {
                    String start = dataRequestDTO.getStart().format(dateTimeFormatter);
                    String end = dataRequestDTO.getEnd().format(dateTimeFormatter);
                    Query publishTime = Query.of(x -> x.range(y -> y.field("publishTime").gte(JsonData.of(start)).lte(JsonData.of(end))));

                    b.must(publishTime);

                }
                // 关键词
                if (!TextUtils.isEmpty(dataRequestDTO.getKeywords())) {
                    b.must(Query.of(y -> y.multiMatch(MultiMatchQuery.of(z -> z.fields(Arrays.asList("title", "content")).query(dataRequestDTO.getKeywords())))));
                    builder.sort(SortOptions.of(so -> so.score(s -> s.order(SortOrder.Desc)))).minScore(10D);
                }

                // 类型
                if (!TextUtils.isEmpty(dataRequestDTO.getCategory())) {
                    b.must(Query.of(x -> x.term(y -> y.field("category").value(dataRequestDTO.getCategory()))));
                }

                // 城市
                if (!TextUtils.isEmpty(dataRequestDTO.getCity())) {
                    List<String> cits = Arrays.stream(dataRequestDTO.getCity().split(",")).filter(x -> !"其他".equals(x) && !"全部".equals(x)).collect(Collectors.toList());
                    for (String ct : cits) {
                        b.must(Query.of(y -> y.match(MatchQuery.of(z -> z.field("city").query(ct)))));
                    }
                }
                return b;
            }));

            builder.query(query);
            builder.sort(SortOptions.of(
                    so -> so.field(fs -> fs.field("publishTime").order(SortOrder.Desc))
            ));

            SearchRequest searchRequest = builder.build();

            response = elasticsearchClient.search(searchRequest, YoumeiEsData.class);
        } catch (IOException e) {
            logger.error("error", e);
        } finally {
            if (elasticsearchClient != null) {
                esClientPool.returnClient(elasticsearchClient);
            }
        }
        List<YoumeiEsData> data = new ArrayList<>();
        long total = 0;
        if (response != null) {
            response.hits().hits().forEach(x -> {
                assert x.source() != null;
                data.add(x.source());
            });
            if (response.hits().total() != null) {
                total = response.hits().total().value();
            }
        }
        page.setTotal(total);
        page.setRecords(data);

        return page;
    }

    public YoumeiEsData getData(String id) {
        try {
            ElasticsearchClient elasticsearchClient = esClientPool.getClient();
            return elasticsearchClient.search(x -> x.index("news_data").query(q -> q.ids(i -> i.values(id))), YoumeiEsData.class).hits().hits().get(0).source();
        } catch (Exception e) {
            throw new BizException(40000, "未知数据");
        }

    }

    public void deleteData(String id) {
        try {
            ElasticsearchClient elasticsearchClient = esClientPool.getClient();
            elasticsearchClient.delete(x -> x.index("news_data").id(id));
        } catch (Exception e) {
            throw new BizException(40000, "删除错误");
        }
    }

    public void putDataRun() {
//        (new Thread(this::putData)).start();
        this.putData();
    }


    public void putData() {
        if(lock){
            throw new BizException(40000,"正在执行，请勿重复执行");
        }
        lock = true;
        log.info("ES 入数据开始");
        YoumeiOffset youmeiOffset = youmeiOffsetService.getById(4);
        long offset = youmeiOffset.getOffset() + 1L;
        long max = 0L;
        ElasticsearchClient elasticsearchClient = null;
        try {
            elasticsearchClient = esClientPool.getClient();
            while (true) {
                List<CategoryData> youmeiDataList = baseMapper.getCategoryList(offset, 1000);
                log.info("本次数据{}条，offset从{}开始", youmeiDataList.size(), offset);
                if (youmeiDataList.size() == 0) {
                    break;
                }
                max = youmeiDataList.get(youmeiDataList.size() - 1).getOffset();
                List<BulkOperation> bulkOperations = new ArrayList<>();

                youmeiDataList.forEach(a -> {
                    YoumeiEsData youmeiEsData = YoumeiEsData.of(a);
                    bulkOperations.add(BulkOperation.of(b -> b.index(c -> c.id(String.valueOf(youmeiEsData.getOffset())).document(youmeiEsData))));
                });

                elasticsearchClient.bulk(BulkRequest.of(x -> x.index("news_data").operations(bulkOperations)));

                if (youmeiDataList.size() < 1000) {
                    break;
                }

                offset += 1000;
            }

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("error", e);
        }finally {
            lock = false;
            if (null != elasticsearchClient){
                esClientPool.returnClient(elasticsearchClient);
            }
            if (max > 0L) {
                youmeiOffset.setOffset(max);
                youmeiOffsetService.updateById(youmeiOffset);
            }
        }

        log.info("ES 入数据结束");
    }
}
