package com.sxtvs.open.api.user.service.impl;

import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.api.user.dto.LoginDto;
import com.sxtvs.open.api.user.dto.PwdResetDTO;
import com.sxtvs.open.api.user.dto.TokenDto;
import com.sxtvs.open.api.user.dto.UserDTO;
import com.sxtvs.open.api.user.entity.Menus;
import com.sxtvs.open.api.user.entity.User;
import com.sxtvs.open.api.user.entity.UserRole;
import com.sxtvs.open.api.user.mapper.UserMapper;
import com.sxtvs.open.api.user.service.IUserService;
import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.auth.AESUtil;
import com.sxtvs.open.core.auth.HttpContextUtil;
import com.sxtvs.open.core.sls.AliyunLogger;
import jakarta.annotation.Resource;
import org.apache.http.util.TextUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zyx
 * @since 2023-02-15
 */
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    private AliyunLogger logger;

    @Resource
    private UserRoleServiceImpl userRoleService;

    @Resource
    private MenusServiceImpl menusService;

    public TokenDto login(LoginDto dto) {

        logger.info("key", "login", "dto", JSON.toJSONString(dto));


        // todo change redis read
        var optionalUser = lambdaQuery().eq(User::getLoginName, dto.getLoginName())
                .eq(User::getLoginPassword, dto.getLoginPassword())
                .oneOpt();

        if (optionalUser.isEmpty()) {
            throw new BizException("用户不存在");
        }

        var id = optionalUser.get().getId();

        var token = AESUtil.encryptHex(id.toString());
        var tokenDto = new TokenDto();
        tokenDto.setToken(token);
        return tokenDto;

    }

    public User getLoginUser() {
        var token = HttpContextUtil.getToken();
        if (token.isEmpty()) {
            throw new RuntimeException("获取token异常");
        }
        var userId = AESUtil.decryptStr(token.get());
        var user = this.lambdaQuery().eq(User::getId, Long.parseLong(userId)).one();
        if (user == null) {
            throw new RuntimeException("用户不存在");
        }

        UserRole userRole = userRoleService.getById(user.getRoleId());

        List<Menus> menus = menusService.lambdaQuery().in(Menus::getId, Arrays.asList(userRole.getMenuId().split(","))).list();
        userRole.setMenus(menus);
        user.setUserRole(userRole);

        return user;
    }

    public User getLoginUserByToken(String token) {
        var userId = AESUtil.decryptStr(token);
        var user = this.lambdaQuery().eq(User::getId, Long.parseLong(userId)).one();
        if (user == null) {
            throw new RuntimeException("用户不存在");
        }
        return user;
    }

    public void createUser(String userName, String pwd) {
        var user = new User();
        user.setLoginName(userName);
        user.setLoginPassword(SecureUtil.sha256(pwd));
        user.setEncodePassword(AESUtil.encryptHex(pwd));
        save(user);
    }

    public void createUser(String userName, String pwd, Integer roleId) {
        var user = new User();
        user.setLoginName(userName);
        user.setLoginPassword(SecureUtil.sha256(pwd));
        user.setEncodePassword(AESUtil.encryptHex(pwd));
        user.setRoleId(roleId);
        save(user);
    }

    public void logout(){
        var token = HttpContextUtil.getToken();
        if (token.isEmpty()) {
            throw new RuntimeException("获取token异常");
        }
        var userId = AESUtil.decryptStr(token.get());
        var user = this.lambdaQuery().eq(User::getId, Long.parseLong(userId)).one();
        if (user == null) {
            throw new RuntimeException("用户不存在");
        }
        user.setLogoutTime(LocalDateTime.now());
        updateById(user);
    }

    public void reset(PwdResetDTO pwdResetDTO){
        Optional<String> token = HttpContextUtil.getToken();

        String uid = AESUtil.decryptStr(token.get());
        User user = this.lambdaQuery().eq(User::getId, Long.parseLong(uid)).eq(User::getLoginPassword, pwdResetDTO.getOldPwd()).one();
        if(null == user){
            throw new BizException(40001, "原密码错误");
        }

        user.setLoginPassword(pwdResetDTO.getPwd());
        user.setEncodePassword("密码已修改");
        updateById(user);

    }

    public Page<User> users(UserDTO userDTO){
        LambdaQueryWrapper<User> lambdaQueryWrapper = Wrappers.lambdaQuery();

        Page<User> userPage = baseMapper.selectPage(new Page<>(userDTO.getPage(), userDTO.getPageSize()),
                lambdaQueryWrapper.like(!TextUtils.isEmpty(userDTO.getName()), User::getLoginName, userDTO.getName())
                        .eq(userDTO.getRoleId() != null && userDTO.getRoleId() > 0, User::getRoleId, userDTO.getRoleId())
        );

        if(userPage.getTotal() > 0){
            userPage.setRecords(userPage.getRecords().stream().peek(user -> {
                user.setUserRole(userRoleService.getById(user.getRoleId()));
            }).collect(Collectors.toList()));
        }

        return userPage;

    }

    public void updateRole(UserDTO userDTO){
        User user = getById(userDTO.getId());
        if(null == user){
            throw new BizException(40001, "不存在的用户");
        }

        user.setRoleId(userDTO.getRoleId());

        updateById(user);
    }
}
